@@ -15,7 +15,7 @@ contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,
 	if(hasArg(leg.txt)) leg.txt<-list(...)$leg.txt
 	else leg.txt<-"trait value"
 	h<-max(nodeHeights(tree))
-	steps<-0:res/res*max(h)
+	steps<-c(0:(res-1)/(res-1)*h,h+h/(res-1)) ## 0:res/res*(h+h/res)
 	H<-nodeHeights(tree)
 	if(method=="fastAnc") a<-fastAnc(tree,x) 
 	else if(method=="anc.ML") { 
@@ -77,15 +77,19 @@ contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,
 	invisible(xx)
 }
 
-# function
-# written by Liam J. Revell 2012
+## internally used function
+## written by Liam J. Revell 2012, 2017
 getState<-function(x,trans){
-	i<-1
-	while(x>trans[i]){
-		state<-names(trans)[i]
-		i<-i+1
+	if(x<=trans[1]) state<-names(trans)[1]
+	else if(x>=trans[length(trans)]) state<-names(trans)[length(trans)]
+	else {
+		i<-1
+		while(x>trans[i]){
+			state<-names(trans)[i]
+			i<-i+1
+		}
 	}
-	return(state)
+	state
 }
 
 ## S3 print method for objects of class "contMap"
