@@ -1,7 +1,8 @@
 ## function for phylogenetic heat map
-## written by Liam J. Revell 2016, 2017
+## written by Liam J. Revell 2016, 2017, 2020
 
 phylo.heatmap<-function(tree,X,fsize=1,colors=NULL,standardize=FALSE,...){
+	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
 	if(length(fsize)!=3) fsize<-rep(fsize,3)
 	if(hasArg(legend)) legend<-list(...)$legend
 	else legend<-TRUE
@@ -24,7 +25,9 @@ phylo.heatmap<-function(tree,X,fsize=1,colors=NULL,standardize=FALSE,...){
 	if(hasArg(mar)) mar<-list(...)$mar
 	else mar<-rep(1.1,4)
 	if(is.null(colors)) colors<-heat.colors(n=20)[20:1]
-	cw<-untangle(tree,"read.tree")
+	if(hasArg(grid)) add.grid <- list(...)$grid
+	else add.grid <- FALSE
+	cw<-untangle(as.phylo(tree),"read.tree")
 	plot.new()
 	par(mar=mar)
 	plot.window(xlim=xlim,ylim=ylim)
@@ -37,6 +40,14 @@ phylo.heatmap<-function(tree,X,fsize=1,colors=NULL,standardize=FALSE,...){
 	image(x=seq(START,END,by=(END-START)/(ncol(X)-1)),
 		z=t(X[cw$tip.label,]),add=TRUE,
 		col=colors,...)
+	if(add.grid){
+	    dx <- (END - START)/(ncol(X) - 1)
+	    x <- seq(START - dx/2, END + dx/2, by = dx) 
+	    nTips <- length(tree$tip.label)
+	    y <- c(-1/(2*(nTips-1)), seq(0, 1, length = nTips) + 1/(2*(nTips-1)) )
+	    segments(x, y[1], x, y[length(y)])
+	    segments(x[1], y, x[length(x)], y)
+	}
 	if(legend) add.color.bar(leg=END-START,cols=colors,lims=range(X,na.rm=TRUE),
 		title=if(standardize) "standardized value" else "value",
 		subtitle=if(standardize) "SD units" else "",prompt=FALSE,x=START,
